#!/bin/sh
zypper=0
licensefile="/etc/op5license/op5license.xml"

if [ ! $1 ]; then
    echo "No repo file supplied"
    exit 1
fi

if [ ! -f $1 ]; then
    echo "File not found:$1"
    exit 1
fi

if [ ! -f $licensefile ]; then
    echo "Licensefile: '$licensefile' not found"
    echo "Do not forget to install your license when you are done."
    exit 0
fi

repo_config="$1"
echo $repo_config|grep zypp >/dev/null
if [ $? -eq 0 ]; then
	zypper=1
fi

# Find auth info in license file
auth=`cat $licensefile | grep updatekey | awk -F\" '{print $2} '`

if [ ! $auth ]; then
    echo "Failed to find auth info in licensefile $licensefile"
    exit 0
fi

# Remove old auth data (if present)
touch /tmp/repoconftmp
sed 's/baseurl=http:\/\/[a-zA-Z0-9\:]*@/baseurl=http:\/\//' $repo_config > /tmp/repoconftmp

if [ $? = 0 ]; then
	cat /tmp/repoconftmp > $repo_config
else
	echo "Repo config failed (sed exited with wrong status)"
fi

rm /tmp/repoconftmp

# Insert new auth data
touch /tmp/repoconftmp
sed "s/baseurl=http:\/\//baseurl=http:\/\/$auth@/" $repo_config > /tmp/repoconftmp

if [ $? = 0 ]; then
	cat /tmp/repoconftmp > $repo_config
	if [ $zypper -eq 1 ]; then
		# Enable and active autorefresh on zypper repo
		for zypper_repo in $(sed -n '/\[.*\]/p' $repo_config |sed -n 's/\[//p'|sed -n 's/\]//p'); do
			zypper mr -r -e $zypper_repo
		done
	fi
else
	echo "Repo config failed (sed exited with wrong status)"
fi

rm /tmp/repoconftmp


exit 0
