<?php
define('OP5LICENSE_CACHE_FILE', './op5license-cache');
define('OP5LICENSE_FILE', '/etc/op5license/op5license.xml');
define('OP5LICENSE_ERR_LICENSE_OK', '1');
define('OP5LICENSE_ERR_EXCEEDS_LICENSE', '0');
define('OP5LICENSE_ERR_UNLICENSED_PRODUCT', '2');
define('OP5LICENSE_TYPE_REGULAR', '1');
define('OP5LICENSE_TYPE_SITE', '2');
define('OP5LICENSE_TYPE_EVALUATION', '3');
define('OP5LICENSE_TYPE_UNLICENSED', '4');
define('OP5LICENSE_TYPE_ENTERPRISE', '5');
define('OP5LICENSE_TYPE_BASIC', '6');
define('OP5LICENSE_TYPE_FREE', '7');
define('OP5LICENSE_CHECK_OK', '1');
define('OP5LICENSE_CHECK_ERROR', '0');
define('OP5LICENSE_CHECK_EVALUATION', '2');
define('OP5LICENSE_CHECK_EVALUATION_EXPIRED', '3');
define('OP5LICENSE_CHECK_ISSUE_DATE', '4');
define('OP5LICENSE_CHECK_EXPIRE_DATE', '5');
define('OP5LICENSE_CHECK_HOSTS_COUNT_EXCEEDS', '6');


class op5license
{
	private $license_file;
	private $license_data;
	public  $license_status = NULL;
	private $license;
	private $verify_output;
	private $verify_return;
	private $products = array();
	private $usage = array();
	private $cache = array();
	private $errors = array();
	private $evaluations = array();
	private $evaluations_expire = array();
	private $monitor_hosts = 20;
	private $logserver_hosts = 10;
	private $license_version = NULL;
	private $allowed_products = array('Monitor');

	public function __construct($override_file = FALSE)
	{
		$file = ($override_file) ? $override_file : OP5LICENSE_FILE;

		if (!$this->set_license_file($file))
			$this->error('License file does not exist. A license file needs to be installed.');

		if (!$this->load_license_from_file())
			$this->error('Invalid license format. The file is not a valid op5 license file, or the file has been tampered with.');

		if (!$this->verify_signature())
			$this->error('Signature does not match license data. The license file is in a valid format, but the signature or the license data has been tampered with.');

		$this->parse_license();
	}


	/**
	 * @return License version
	 */
	public function get_license_version()
	{
		return $this->license_version;
	}

	/**
	 * Make reading $this->license_data[$key] compatible for op5v1 and op5v2
	 * 
	 * @param $key key of $this->license_data array
	 * @param $prefix used for nested arrays
	 * 
	 * @return str   
	 */
	private function license_data($key, $prefix = '')
	{
		//echo $this->license_version;	
		//var_dump($this->license_data);
		if ($this->license_version === 'op5v2') {
			switch($prefix) {
			 case 'monitor':
				if (isset($this->license_data['products'][$prefix][$key]))
					return $this->license_data['products'][$prefix][$key];
				return '';
			 case 'logserver':
				return '';
			 case 'extensions':
				if (isset($this->license_data[$prefix][$key]))
					return $this->license_data[$prefix][$key];
				return '';
			}
		}
		if (isset($this->license_data[$key]))
			return $this->license_data[$key];
		return '';
	}


	private function isset_license_data($key, $preffix = '')
	{
			
		switch ($this->license_version) {
			case 'op5v2':
				if($preffix == '') return isset($this->license_data[$key]);
				
				switch($preffix)
				{
					case 'monitor':
					case 'logserver':
						return isset($this->license_data['products'][$preffix][$key]);
						break;
					case 'extensions':
						return isset($this->license_data[$preffix][$key]);
						break;
				}
				break;
			
			default:
				return isset($this->license_data[$key]);
				break;
		}
		
	}


	private function op5license_Monitor()
	{
		$fp = @fopen("/opt/monitor/etc/nagios.cfg", "r");
		if ($fp === false) return -1;

		do {
			$line = @fgets($fp);
		} while($line !== false && !preg_match( "/^[\s]*broker_module=.*livestatus\.o.*[\s]([^\s]+)[\s]*$/", $line, $matches ));

		@fclose($fp);

		if( $line === false ) return -2;
		$livestatus_path = $matches[1];

		$query = "GET hosts\nStats: state != 999\n\n";

		$conn = @fsockopen('unix://'.$livestatus_path, NULL, $errno, $errstr, 2);
		if( $conn === false ) return -3;
		$out  = @fwrite($conn, $query);
		if( $out === false ) return -4;
		$count = intval(@fgets($conn));
		if( $count === false ) return -5;
		@fclose($conn);

		return $count;
	}

	private function op5license_Logserver()
	{
		$logserver_cache_file = '/tmp/logserver-license.dat';

		if (!file_exists($logserver_cache_file))
			return -1;

		$fp = fopen($logserver_cache_file, 'r');
		$in = fread($fp, filesize($logserver_cache_file));
		$in = rtrim($in);

		$ary = explode(':', $in);

		$num = count($ary);

		return $num;
	}

	private function error($message, $product = NULL)
	{
		$this->errors[isset($product) ? $product : 'General'][] = $message;
	}

	private function set_license_file($file)
	{
		return !is_null($this->license_file = file_exists($file) ? $file : NULL);
	}

	public function get_license_file()
	{
		return ($this->license != NULL) ? $this->license : FALSE;
	}

	public function license_file_exists()
	{
		return is_file($this->license_file);
	}

	private function verify_signature()
	{
		exec("/opt/op5sys/bin/op5license-verify {$this->license_file}", $verify_output, $verify_return);
		return (bool)($verify_return == 0);
	}

	private function load_license_from_file()
	{
		if ($this->license_file) {
			$license = @simplexml_load_file($this->license_file);
			if ((bool)($license)) {
				$this->license = $license;
				return true;
			}
		}
		return false;
	}

	private function parse_license()
	{
		
		if (isset($this->license))
		{
			foreach ($this->license->attributes() as $key => $val) {
				if($key == 'verifier')
				{
					$this->license_version = $val;
					break;
				}
			}
			
			
			foreach ($this->license->item as $item)
			{
				$this->license_data[(string)$item['name']] = (string)$item['value'];
				//for special case: <item name="LicenseType" value="Evaluation" lifetime="30"></item>
				if($item['name'] == 'LicenseType' && $item['value'] == 'Evaluation' && isset($item['lifetime']))
				{
					$this->license_data['lifetime'] = (int)$item['lifetime'];
				}
				
			}
			
			//for op5v2
			// now we have one product - "Monitor", other "stuff-things" are now extensoins for monitor
			if($this->license->products)
			{
				$this->license_data['products'] = array();
				
				foreach ($this->license->products->monitor->item as $item)
				{
					$this->license_data['products']['monitor'][(string)$item['name']] = (string)$item['value']; 
				}
				
				//extensions for monitor
				if($this->license->products->monitor->extensions)
				{
					$this->license_data['extensions'] = array();
					foreach ($this->license->products->monitor->extensions->item as $item)
					{
						$this->license_data['extensions'][(string)$item['name']]['name'] = (string)$item['name'];
						$this->license_data['extensions'][(string)$item['name']]['value'] = (string)$item['value'];
						if(isset($item['Hosts']))
						 {
							$this->license_data['extensions'][(string)$item['name']]['Hosts'] = (string)$item['Hosts'];
						} 
					}	
				}		
				
			}
			
			
			return TRUE;
		}
		else
			return FALSE;
	}

	public function register_product($product_name, $override_usage_callback = NULL)
	{
		// only for op5v2
		if($this->get_license_version() == 'op5v2' && !in_array($product_name, $this->allowed_products) )
		{
			return;
		}
		
		if (isset($override_usage_callback))
			$this->products[$product_name] = $override_usage_callback;
		else
		{
			$callback = 'op5license_' . $product_name;
			$this->products[$product_name] = $callback;
			require_once($callback . '.php');
		}
	}

	private function check_monitor_product_type() 
	{
		/* If MonitorProductType is set in the license we use it to decide the type */
		if ( $this->isset_license_data('MonitorProductType','monitor')) {
			switch ($this->license_data('MonitorProductType','monitor')) {
			 case 'Enterprise': 
				return 'Enterprise';
			 case 'Basic':
				return 'Basic';
			 default:
				return 'Free';
			}
		}
		/* If MonitorProductType is not set we us MonitorLicenseType. Regular & Site is considered Enterprise */
		switch ($this->license_data('MonitorLicenseType','monitor'))
		{	
		 case 'Regular':
		 case 'Site':
		 case 'Enterprise':
			return 'Enterprise';
		 case 'Basic':
			return 'Basic';
		}
		return 'Free';
	}



	private function check_product_license_type($product)
	{
		
		switch ($this->license_data($product . 'LicenseType', strtolower($product)))
		{
			case 'Regular':
				return OP5LICENSE_TYPE_REGULAR;

			case 'Site':
				return OP5LICENSE_TYPE_SITE;

			case 'Evaluation':
				return OP5LICENSE_TYPE_EVALUATION;

			case 'Enterprise':
				return OP5LICENSE_TYPE_ENTERPRISE;

			case 'Basic':
				return OP5LICENSE_TYPE_BASIC;

			case 'Free':
				return OP5LICENSE_TYPE_FREE;

			default:
				return OP5LICENSE_TYPE_UNLICENSED;
		}
		

	}

	

	private function check_product_usage($product, $callback)
	{
		//echo $this->license_version,' | ',$product,' | ', $callback, '<br />';
		//since APS4 with op5v2 we have only one product "Monitor" with couple of extensions
		if($this->license_version == 'op5v2' && strtolower($product) !== 'monitor') return OP5LICENSE_ERR_LICENSE_OK;

		if (!isset($this->usage[$product]))
			$this->usage[$product] = call_user_func($callback);

		if ($this->check_product_license_type($product) == OP5LICENSE_TYPE_REGULAR || 
			$this->check_product_license_type($product) == OP5LICENSE_TYPE_ENTERPRISE ||
			$this->check_product_license_type($product) == OP5LICENSE_TYPE_BASIC)
		{
			$license_data_string = $product . 'Hosts';

			if (!$this->get_max_hosts_usage($product))
				return OP5LICENSE_ERR_UNLICENSED_PRODUCT;
			elseif($this->usage[$product] <= $this->get_max_hosts_usage($product))
				return OP5LICENSE_ERR_LICENSE_OK;
			else
				return OP5LICENSE_ERR_EXCEEDS_LICENSE;
		}
		elseif ($this->check_product_license_type($product) == OP5LICENSE_TYPE_UNLICENSED)
		{
			return OP5LICENSE_ERR_UNLICENSED_PRODUCT;
		}
		else
			return OP5LICENSE_ERR_LICENSE_OK;
	}

	public function get_customer_info()
	{
		$customer['CustomerName'] = $this->license_data['IssuedToName'];
		$customer['CustomerEmail'] = $this->license_data['IssuedToEmail'];
		$customer['LicenseIssueDate'] = $this->license_data['IssueDate'];
		if (substr($this->license_data['IssuedToCustomerNumber'], -3, 1) == '-')
		{
			$customer['CustomerNumber'] = substr($this->license_data['IssuedToCustomerNumber'], 0, -3);
			$customer['LicenseKeyNumber'] = substr($this->license_data['IssuedToCustomerNumber'], -2);
		}
		else
			$customer['CustomerNumber'] = $this->license_data['IssuedToCustomerNumber'];

		//for op5v2
		if(isset($this->license_data['Evaluation'])) $customer['LicenseEvaluation'] = $this->license_data['Evaluation'];
		if(isset($this->license_data['ExpireDate'])) $customer['LicenseExpireDate'] = $this->license_data['ExpireDate'];
		if(isset($this->license_data['hosts'])) $customer['LicenseHosts'] = $this->license_data['hosts'];
		if(isset($this->license_data['products'])) $customer['LicenseProducts'] = $this->license_data['products'];
		if(isset($this->license_data['extensions'])) $customer['LicenseExtensions'] = $this->license_data['extensions'];
		

		return $customer;
	}

	public function get_errors()
	{
		return $this->errors;
	}

	public function get_products()
	{
		if (count($this->usage) < count($this->products))
			$this->check();

		$p = array();
		foreach ($this->usage as $product => $current_usage)
		{
			$p[$product]['name'] = $product;
			$p[$product]['license_type'] = $this->check_product_license_type($product);
			$p[$product]['product_type'] = $this->check_monitor_product_type();
			$p[$product]['current_usage'] = $current_usage;
			switch ($p[$product]['license_type'])
			{
				case OP5LICENSE_TYPE_REGULAR:
					$p[$product]['license_type_str'] = 'Regular';
					$p[$product]['max_usage'] = $this->get_max_hosts_usage($product);
					break;
				case OP5LICENSE_TYPE_ENTERPRISE:
					$p[$product]['license_type_str'] = 'Enterprise';
					$p[$product]['max_usage'] = $this->get_max_hosts_usage($product);
					break;
				case OP5LICENSE_TYPE_BASIC:
					$p[$product]['license_type_str'] = 'Basic';
					$p[$product]['max_usage'] = $this->get_max_hosts_usage($product);
					break;

				case OP5LICENSE_TYPE_SITE:
					$p[$product]['license_type_str'] = 'Site';
					break;

				case OP5LICENSE_TYPE_EVALUATION:
					$p[$product]['license_type_str'] = 'Evaluation';
					break;
				case OP5LICENSE_TYPE_FREE:
					$p[$product]['license_type_str'] = 'Free';
					break;

				default:
					$p[$product]['license_type_str'] = 'Unlicensed';
			}

			if (!empty($this->errors[$product]))
				$p[$product]['errors'] = $this->errors[$product];
		}

		return $p;
	}

	public function check($check_usage = TRUE)
	{
		if (count($this->products) > 0)
			foreach ($this->products as $product => $callback)
			{
				$type = $this->check_product_license_type($product);
				switch ($product)
				{
					case 'Monitor' :
						$number_of_hosts = $this->op5license_Monitor();
						$max_hosts = $this->monitor_hosts;
						break;

					case 'Logserver' :
						$number_of_hosts = $this->op5license_Logserver();
						$max_hosts = $this->logserver_hosts;
						break;
				}

				if ($check_usage != FALSE)
				{
					$usage = $this->check_product_usage($product, $callback);
				}
				else
				{
					$usage = OP5LICENSE_ERR_LICENSE_OK;
					$this->usage[$product] = 0;
				}

				if ($usage == OP5LICENSE_ERR_EXCEEDS_LICENSE)
					$this->error('Usage exceeds license!', $product);
				elseif ($usage == OP5LICENSE_ERR_UNLICENSED_PRODUCT)
				{
					$this->error('This product is unlicensed.', $product);
				}
				if ($type == OP5LICENSE_TYPE_EVALUATION)
				{
						
					$this->evaluations[$product] = 1;
					
					if(strtotime($this->license_data('IssueDate', strtolower($product))) < time())
					{
						$this->evaluations_expire[$product] = 1;
					}
				}
			}
		else
		{
			$number_of_hosts = 0;
			$max_hosts = 0;
		}


		//here we have all info about products, etc
		
		
		
		if (count($this->errors) && $this->license_file)
		{
			return $this->license_status = OP5LICENSE_CHECK_ERROR;
		}
		elseif(isset($this->license_data['hosts']) && ($this->amount_hosts() > $this->license_data['hosts']))
		{
			return $this->license_status = OP5LICENSE_CHECK_HOSTS_COUNT_EXCEEDS;		
		}
		elseif($this->isset_license_data('IssueDate') && strtotime($this->license_data('IssueDate')) > time())
		{
			return $this->license_status = OP5LICENSE_CHECK_ISSUE_DATE;
		}
		elseif(file_exists(OP5LICENSE_FILE) && $this->isset_license_data('IssueDate') && $this->isset_license_data('IssueDate') && $this->isset_license_data('LicenseType') && $this->license_data('LicenseType') == 'Evaluation' && ((strtotime($this->license_data('ExpireDate'))-strtotime($this->license_data('IssueDate')))/(3600*24) == 60) ) //implemented on customer's demand 
		{
			if(strtotime($this->license_data('ExpireDate')) < time() )
			{
				 if($this->check_evluation_lifetime()) return $this->license_status = OP5LICENSE_CHECK_OK;
			}
			return $this->license_status = OP5LICENSE_CHECK_EXPIRE_DATE;
		}
		elseif($this->isset_license_data('ExpireDate') && strtotime($this->license_data('ExpireDate')) < time())
		{
			return $this->license_status = OP5LICENSE_CHECK_EXPIRE_DATE;
		}
		elseif (count($this->evaluations) && $number_of_hosts > $max_hosts) //done on customer's demand
		{
			if(count($this->evaluations_expire))
				return $this->license_status = OP5LICENSE_CHECK_EVALUATION_EXPIRED;
			else
				return $this->license_status = OP5LICENSE_CHECK_EVALUATION;
		}
		elseif(count($this->evaluations) && count($this->evaluations_expire))
		{
			return $this->license_status = OP5LICENSE_CHECK_EVALUATION_EXPIRED;
		}
		elseif (!$this->license_file && $number_of_hosts > $max_hosts)
		{
			return $this->license_status = OP5LICENSE_CHECK_ERROR;
		}
		else
		{
			return $this->license_status = OP5LICENSE_CHECK_OK;
		}
	}

	public function check_Monitor_allowed()
	{
		if ($this->check_product_license_type('Monitor') == OP5LICENSE_TYPE_SITE){
			return 0;
		}
		if($this->isset_license_data('MonitorHosts', 'monitor'))
			return $this->license_data('MonitorHosts', 'monitor');
		return 0;
	}

	public function status()
	{
		if ($this->license_status == NULL)
		  $this->check();

		return $this->license_status;
	}

	private function license_bar()
	{
		if ($this->license_status == OP5LICENSE_CHECK_OK)
			return NULL;

		$license_info = '<div id="licenseInfo" onclick="javascript:top.location.href=\'/license.php\'" style="background-color: #333333; height: 24px; padding: 18px 0px; font-size: 16px">';
		if ($this->license_status == OP5LICENSE_CHECK_ERROR)
			$license_info .= '<strong>You have an invalid license.</strong>';
		elseif ($this->license_status == OP5LICENSE_CHECK_EVALUATION)
			$license_info .= '<strong>You have an evaluation license.</strong>';
		elseif ($this->license_status == OP5LICENSE_CHECK_EVALUATION_EXPIRED)
			$license_info .= '<strong>Your evaluation license has expired.</strong>';
		elseif ($this->license_status == OP5LICENSE_CHECK_HOSTS_COUNT_EXCEEDS)
			$license_info .= '<strong>Hosts limit exceeded.</strong>';	
		elseif ($this->license_status == OP5LICENSE_CHECK_ISSUE_DATE)
			$license_info .= '<strong>Your issue date not started yet.</strong>';
		elseif ($this->license_status == OP5LICENSE_CHECK_EXPIRE_DATE)
			$license_info .= '<strong>Your license has expired.</strong>';		
			
		$license_info .= '<span style="font-family: arial; padding-left: 10px; color: #ffffff">To get access to updates via op5 and the op5 support services, you need to purchase a valid license. Click here for more information.</span></div>';
		
		return $license_info;
	}

	public function print_license_bar()
	{
		echo $this->license_bar();
	}
	
	/**
	 * @return Count of hosts used by monitor and logserver
	 */
	public function amount_hosts()
	{
		$monitor_hosts = (int)$this->op5license_Monitor();
		$logserver_hosts = (int)$this->op5license_Logserver();
		if($monitor_hosts < 0) $monitor_hosts=0;
		if($logserver_hosts < 0) $logserver_hosts=0;
		
		return $monitor_hosts+$logserver_hosts;
		
	}
	
	/**
	 * Check if lifetime of evaluation license hasn't expired
	 * True if license Ok
	 * False if license expired by lifetime
	 * 
	 * @return bool
	 */
	private function check_evluation_lifetime()
	{
		$time = (time() - filemtime(OP5LICENSE_FILE))/(60*60*24);
		return (bool)($this->license_data['lifetime'] > $time);
	}
	
	
	private function get_max_hosts_usage($product)
	{
		if($this->isset_license_data($product . 'Hosts', strtolower($product)))
			return $this->license_data($product . 'Hosts', strtolower($product));
		
		if($this->isset_license_data('hosts'))
			return $this->license_data('hosts');
		
		return 0; //if no info about hosts we assume it's 0
		
	}
}
?>
